@@ -328,6 +328,8 @@ static const int line_break_indexes[] = {
   INDEX_WORD_JOINER
 };
 
+#define BREAK_TYPE_SAFE(btype)            \
+         (btype < G_N_ELEMENTS(line_break_indexes) ? btype : G_UNICODE_BREAK_UNKNOWN)
 #define BREAK_INDEX(btype)                \
          (line_break_indexes[(btype)])
 #define BREAK_ROW(before_type)            \
@@ -335,7 +337,7 @@ static const int line_break_indexes[] = {
 #define BREAK_OP(before_type, after_type) \
          (BREAK_ROW (before_type)[BREAK_INDEX (after_type)])
 #define IN_BREAK_TABLE(btype)             \
-         (btype <= G_N_ELEMENTS(line_break_indexes) && BREAK_INDEX(btype) < INDEX_END_OF_TABLE)
+         (btype < G_N_ELEMENTS(line_break_indexes) && BREAK_INDEX(btype) < INDEX_END_OF_TABLE)
 
 /* Keep these in sync with the same macros in the test program */
 
@@ -491,6 +493,7 @@ pango_default_break (const gchar   *text,
     next_wc = '\n';
 
   next_break_type = g_unichar_break_type (next_wc);
+  next_break_type = BREAK_TYPE_SAFE (next_break_type);
 
   for (i = 0; i <= n_chars; i++)
     {
@@ -529,6 +532,7 @@ pango_default_break (const gchar   *text,
             }
 	  
 	  next_break_type = g_unichar_break_type (next_wc);
+          next_break_type = BREAK_TYPE_SAFE (next_break_type);
         }
 
       type = g_unichar_type (wc);
@@ -1411,12 +1415,20 @@ pango_find_paragraph_boundary (const gchar *text,
   const gchar *end;
   const gchar *start = NULL;
   const gchar *delimiter = NULL;
-  gunichar prev_wc;
 
   /* Only one character has type G_UNICODE_PARAGRAPH_SEPARATOR in
-   * Unicode 3.0; update this if that changes.
+   * Unicode 4.1; update the following code if that changes.
    */
+
+  /* prev_sep is the first byte of the previous separator.  Since
+   * the valid separators are \r, \n, and PARAGRAPH_SEPARATOR, the
+   * first byte is enough to identify it.
+   */
+  gchar prev_sep;
+
+
 #define PARAGRAPH_SEPARATOR 0x2029
+#define PARAGRAPH_SEPARATOR_STRING "\xE2\x80\xA9"
   
   if (length < 0)
     length = strlen (text);
@@ -1432,29 +1444,21 @@ pango_find_paragraph_boundary (const gchar *text,
   if (length == 0)
     return;
 
-  /* FIXME there's plenty of room to optimize this; e.g. there's
-   * no real need to g_utf8_get_char() on every char
-   */
-  
-  prev_wc = 0;
+  prev_sep = 0;
 
   while (p != end)
     {
-      gunichar wc;
-
-      wc = g_utf8_get_char (p);
-      
-      if (prev_wc == '\n' ||
-          prev_wc == PARAGRAPH_SEPARATOR)
+      if (prev_sep == '\n' ||
+          prev_sep == PARAGRAPH_SEPARATOR_STRING[0])
         {
           g_assert (delimiter);
           start = p;
           break;
         }
-      else if (prev_wc == '\r')
+      else if (prev_sep == '\r')
         {
           /* don't break between \r and \n */
-          if (wc != '\n')
+          if (*p != '\n')
             {
               g_assert (delimiter);
               start = p;
@@ -1462,13 +1466,18 @@ pango_find_paragraph_boundary (const gchar *text,
             }
         }
       
-      if ((wc == '\n' ||
-           wc == '\r' ||
-           wc == PARAGRAPH_SEPARATOR) &&
-          delimiter == NULL)
-        delimiter = p;
+      if (*p == '\n' ||
+           *p == '\r' ||
+           !strncmp(p, PARAGRAPH_SEPARATOR_STRING,
+		    strlen(PARAGRAPH_SEPARATOR_STRING)))
+        {
+          if (delimiter == NULL)
+	    delimiter = p;
+	  prev_sep = *p;
+	}
+      else
+	prev_sep = 0;
       
-      prev_wc = wc;
       p = g_utf8_next_char (p);
     }
 
